@@ -5,13 +5,15 @@ from napari_plugin_engine import napari_hook_implementation
 
 import napari
 from napari_time_slicer import time_slicer
-from napari_tools_menu import register_function
+from napari_tools_menu import register_function, register_dock_widget
+from magicgui import magic_factory
 
 @napari_hook_implementation
 def napari_experimental_provide_function():
     return [
         Train_object_segmentation,
         Apply_object_segmentation,
+        Train_probability_mapper,
         Apply_probability_mapper,
         Train_object_segmentation_from_visible_image_layers,
         Apply_object_segmentation_to_visible_image_layers,
@@ -19,7 +21,7 @@ def napari_experimental_provide_function():
         Apply_pixel_classification,
         Train_pixel_classifier_from_visible_image_layers,
         Apply_pixel_classification_to_visible_image_layers,
-        Connected_component_labeling, Train_object_classifier, Apply_object_classification]
+        Connected_component_labeling, Apply_object_classification]
 
 def Train_pixel_classifier(
         image: "napari.types.ImageData",
@@ -40,6 +42,26 @@ def Train_pixel_classifier(
     result = clf.predict(features=feature_stack, image=image)
     return result
 
+def Train_probability_mapper(
+        image: "napari.types.ImageData",
+        annotation : "napari.types.LabelsData",
+        model_filename : str = "ProbabilityMapper.cl",
+        featureset : PredefinedFeatureSet = PredefinedFeatureSet.small_quick,
+        custom_features : str = "original gaussian_blur=1 sobel_of_gaussian_blur=1",
+        output_probability_of_class : int = 2,
+        max_depth : int = 2,
+        num_ensembles : int = 10
+) -> "napari.types.LabelsData":
+    feature_stack = featureset.value
+    if feature_stack == "":
+        feature_stack = custom_features
+
+    clf = ProbabilityMapper(opencl_filename=model_filename, num_ensembles=num_ensembles, max_depth=max_depth, output_probability_of_class=output_probability_of_class)
+    clf.train(feature_stack, annotation, image)
+
+    result = clf.predict(features=feature_stack, image=image)
+    return result
+
 @register_function(menu="Segmentation / labeling > Semantic segmentation (apply pretrained, APOC)")
 @time_slicer
 def Apply_pixel_classification(image: "napari.types.ImageData",
@@ -47,7 +69,9 @@ def Apply_pixel_classification(image: "napari.types.ImageData",
                                viewer: napari.Viewer = None) -> "napari.types.LabelsData":
 
     clf = PixelClassifier(opencl_filename=model_filename)
-    result = clf.predict(image=image)
+    print("Hello world")
+    result = clf.predict(image=[image])
+    print("Result is ", result.shape, result.dtype)
     return result
 
 def Train_pixel_classifier_from_visible_image_layers(
@@ -96,9 +120,9 @@ def Train_object_segmentation(
         feature_stack = custom_features
 
     clf = ObjectSegmenter(opencl_filename=model_filename, num_ensembles=num_ensembles, max_depth=max_depth, positive_class_identifier=annotated_object_intensity)
-    clf.train(feature_stack, annotation, image)
+    clf.train(feature_stack, annotation, [image])
 
-    result = clf.predict(feature_stack, image)
+    result = clf.predict(feature_stack, [image])
     return result
 
 
@@ -118,7 +142,7 @@ def Apply_object_segmentation(image: "napari.types.ImageData",
                               model_filename : str = "ObjectSegmenter.cl",
                               viewer: napari.Viewer = None) -> "napari.types.LabelsData":
     clf = ObjectSegmenter(opencl_filename=model_filename)
-    result = clf.predict(image=image)
+    result = clf.predict(image=[image])
     return result
 
 def Train_object_segmentation_from_visible_image_layers(
@@ -163,56 +187,63 @@ def Connected_component_labeling(labels: "napari.types.LabelsData", object_class
         instances = cle.connected_components_labeling_box(binary)
     return instances
 
-
+@register_dock_widget(menu="Segmentation post-processing > Object classification (APOC)")
+@magic_factory(
+    model_filename=dict(widget_type='FileEdit', mode='w'),
+    shape_extension_ratio=dict(label='shape (extension ratio)')
+)
 def Train_object_classifier(image: "napari.types.ImageData",
-        labels : "napari.types.LabelsData",
-        annotation : "napari.types.LabelsData",
-        model_filename : str = "ObjectClassifier.cl",
-        max_depth : int = 2,
-        num_ensembles : int = 10,
-        area : bool = True,
-        min_intensity: bool = False,
-        mean_intensity: bool = False,
-        max_intensity: bool = False,
-        sum_intensity: bool = False,
-        standard_deviation_intensity: bool = False,
-        shape: bool = False,
-        position:bool = False,
-        touching_neighbor_count:bool = False,
-        average_distance_of_touching_neighbors:bool = False,
-        distance_to_nearest_neighbor:bool = False,
-        average_distance_to_6_nearest_neighbors:bool = False,
-        average_distance_to_10_nearest_neighbors:bool = False,
-    ) -> "napari.types.LabelsData":
+                            labels : "napari.types.LabelsData",
+                            annotation : "napari.types.LabelsData",
+                            model_filename : "magicgui.types.PathLike" = "ObjectClassifier.cl",
+                            max_depth : int = 2,
+                            num_ensembles : int = 10,
+                            minimum_intensity: bool = False,
+                            mean_intensity: bool = False,
+                            maximum_intensity: bool = False,
+                            sum_intensity: bool = False,
+                            standard_deviation_intensity: bool = False,
+                            pixel_count: bool = True,
+                            shape_extension_ratio: bool = False,
+                            centroid_position:bool = False,
+                            touching_neighbor_count:bool = False,
+                            average_centroid_distance_of_touching_neighbors:bool = False,
+                            centroid_distance_to_nearest_neighbor:bool = False,
+                            average_centroid_distance_to_6_nearest_neighbors:bool = False,
+                            average_centroid_distance_to_10_nearest_neighbors:bool = False,
+                            show_classifier_statistics = False,
+                            viewer : "napari.Viewer" = None
+                            ) -> "napari.types.LabelsData":
 
     features = ","
-    if area:
+    if pixel_count:
         features = features + "area,"
-    if min_intensity:
+    if minimum_intensity:
         features = features + "min_intensity,"
     if mean_intensity:
         features = features + "mean_intensity,"
-    if max_intensity:
+    if maximum_intensity:
         features = features + "max_intensity,"
-    if sum_intensity:
-        features = features + "sum_intensity,"
     if standard_deviation_intensity:
         features = features + "standard_deviation_intensity,"
-    if shape:
+    if sum_intensity:
+        features = features + "sum_intensity,"
+    if shape_extension_ratio:
         features = features + "mean_max_distance_to_centroid_ratio,"
-    if position:
+    if centroid_position:
         features = features + "centroid_x,centroid_y,centroid_z,"
     if touching_neighbor_count:
         features = features + "touching_neighbor_count,"
-    if average_distance_of_touching_neighbors:
+    if average_centroid_distance_of_touching_neighbors:
         features = features + "average_distance_of_touching_neighbors,"
-    if distance_to_nearest_neighbor:
+    if centroid_distance_to_nearest_neighbor:
         features = features + "average_distance_of_n_nearest_neighbors=1,"
-    if average_distance_to_6_nearest_neighbors:
+    if average_centroid_distance_to_6_nearest_neighbors:
         features = features + "average_distance_of_n_nearest_neighbors=6,"
-    if average_distance_to_10_nearest_neighbors:
+    if average_centroid_distance_to_10_nearest_neighbors:
         features = features + "average_distance_of_n_nearest_neighbors=10,"
 
+    # remove first and last comma
     features = features[1:-1]
 
     clf = ObjectClassifier(opencl_filename=model_filename, num_ensembles=num_ensembles,
@@ -220,6 +251,14 @@ def Train_object_classifier(image: "napari.types.ImageData",
 
     clf.train(features, labels, annotation, image)
     result = clf.predict(labels, image)
+
+    if show_classifier_statistics and viewer is not None:
+        from qtpy.QtWidgets import QTableWidget
+        from ._dock_widget import update_model_analysis
+        table = QTableWidget()
+        update_model_analysis(table, clf)
+        viewer.window.add_dock_widget(table)
+    
     return result
 
 @register_function(menu="Segmentation post-processing > Object classification (apply pretrained, APOC)")
